@@ -131,7 +131,7 @@ class DexedAudioProcessor  : public AudioProcessor, public AsyncUpdater, public
     void unpackOpSwitch(char packOpValue);
     void packOpSwitch();
 
-    float dpiScaleFactor = -1;
+    float zoomFactor = 1;
 
 public :
     // in MIDI units (0x4000 is neutral)
@@ -280,9 +280,9 @@ public :
     
     std::string currentSCLData = "";
     std::string currentKBMData = "";
-    void setDpiScaleFactor(float factor);
-    float getDpiScaleFactor() {
-        return dpiScaleFactor;
+    void setZoomFactor(float factor);
+    float getZoomFactor() {
+        return zoomFactor;
     }    
 private:
     int chooseNote(uint8_t pitch);
